<?php

require_once(dirname(__FILE__)."/handler/handler_users.php");
require_once(dirname(__FILE__)."/handler/handler_items.php");
require_once(dirname(__FILE__)."/handler/handler_tests.php");
require_once(dirname(__FILE__)."/handler/handler_assessments.php");
require_once(dirname(__FILE__)."/objects/BaseItem.php");
require_once(dirname(__FILE__)."/users.php");
require_once(dirname(__FILE__)."/items.php");
require_once(dirname(__FILE__)."/categories.php");
require_once(dirname(__FILE__)."/tags.php");
require_once(dirname(__FILE__)."/tests.php");
require_once(dirname(__FILE__)."/assessments.php");
require_once(dirname(__FILE__)."/importer.php");
require_once(dirname(__FILE__)."/exporter.php");


class Tester {
    
    public $context;
    public $navigation;
    public $items;
    public $tests;
    public $assessments;
    
    function __construct($page = false) {
        global $TeKe;
        $TeKe->translator->addDomain("tester", 'i18n');
        $TeKe->translator->useDomain("tester");
        $TeKe->template->setTemplateRepository(dirname(__FILE__).'/views/templates');
        $TeKe->template->setTemplateRepository(dirname(__FILE__).'/views/macros');
        $TeKe->template->setTemplateRepository(dirname(__FILE__).'/views/users');
        $TeKe->template->setTemplateRepository(dirname(__FILE__).'/views/items');
        $TeKe->template->setTemplateRepository(dirname(__FILE__).'/views/tests');
        $TeKe->template->setTemplateRepository(dirname(__FILE__).'/views/assessments');
        if (count($page)>0) {
            $this->loadContext($page);
        }
        $this->setNavigation($TeKe);
        $this->users = new Users();
        $this->items = new Items();
        $this->categories = new Categories();
        $this->tags = new Tags();
        $this->tests = new Tests();
        $this->assessments = new Assessments();
    }
    
    function setNavigation($teke) {
        if ($teke->is_admin()) {
            $teke->navigation['users'] = array('title'=>_('Users'), 'url'=>"users/view", 'current'=>$teke->is_current_main('users'), 'level'=>0);
            $teke->navigation['items'] = array('title'=>_('Items'), 'url'=>"items/view", 'current'=>$teke->is_current_main('items'), 'level'=>0);
            $teke->navigation['tests'] = array('title'=>_('Tests'), 'url'=>"tests/view", 'current'=>$teke->is_current_main('tests'), 'level'=>0);
            $teke->navigation['assessments'] = array('title'=>_('Assessments'), 'url'=>"assessments/view", 'current'=>$teke->is_current_main('assessments'), 'level'=>0);
        }    
    }
    
    function loadType($type, $id = false) {
        if (!$type && is_numeric($id) && $id > 0) {
            $type_q = query_row("SELECT type FROM " . DB_PREFIX . "base_item WHERE id=".$id);
            $type = $type_q->type;
        }
        $type = lcfirst($type);
        if ($type) {
            if (is_file(dirname(__FILE__)."/objects/".$type.".php")) {
                require_once(dirname(__FILE__)."/objects/".$type.".php");
                $classname = ucfirst($type);
                return new $classname($id);  
            }
        }
        return false;  
    }
    
    function deleteType($type, $id) {
        if (!$type && is_numeric($id) && $id > 0) {
            $type_q = query_row("SELECT type FROM " . DB_PREFIX . "base_item WHERE id=".$id);
            $type = $type_q->type;
        }
        if ($type) {
            if (is_file(dirname(__FILE__)."/objects/".$type.".php")) {
                require_once(dirname(__FILE__)."/objects/".$type.".php");
                $obj = new $type($id);
                return $obj->delete();
            }
        }
        return false;  
    }
    
    function loadContext($page) {
        if (count($page)>=2 && is_numeric($page[1])) {
            if ($page[0] == "user") {
                //global $TeKe;
                //$this->context = $TeKe->user->getUserById($page[1]);
                $this->context = query_row("SELECT * FROM ". DB_PREFIX ."users WHERE id=".$page[1]);
            } else {
                $this->context = $this->loadType($page[0], $page[1]);
            }
        } else if (count($page)>1) {
            $this->context = $this->loadType($page[0], false);
        }
    }

    function formatDate($date, $format="d.m.Y") {
        if (!$date) {
            return "";
        }
        $time = strtotime($date);
        return date($format, $time);
    }

    function is_test_constructor() {
        global $TeKe;
        return $TeKe->has_access(5);
    }

    /**
     * Used in top of actions, for admins and test constructors only. 
     * All others will be forwarded to index.
     */
    function test_constructor_gatekeeper() {
        if (!$this->is_test_constructor()) {
            forward("index");
        }
    }

    function getSiteLanguages() {
        return array(
            "en" => "english", 
            "et" => "eesti", 
            "ru" => "русский", 
        );
    }

    function getRoles() {
        return array(
            5 => _("teacher"),
            9 => _("admin"),
        );
    }

    function getDifficulties() {
        return array(
            1 => _("beginner"),
            2 => _("intermediate"),
            3 => _("advanced"),   
        );
    }

    function calculatePercent($score, $max_score) {
        if ($max_score == 0) {
            return 100;
        }
        $percent = ($score / $max_score) * 100;
        if ($percent > 100)
            return 100;
        if ($percent < 0)
            return 0;
        return round($percent); 
    }

}
?>
